/******************************************************************************************
 * To add additional script to the scripting widget, create/upload a JavaScript file
 * to the 'scripts' folder. Open the scripting.html file in the editor by double clicking
 * the file in the Project view. Append the filename to the 'scripts' attribute in the
 * ti-widget-scripting element, separated each script filename by a comma.
 ******************************************************************************************/

const SCRIPT_START_MESSAGE = 'Script Started';
const SCRIPT_END_MESSAGE   = 'Script Ended';
const PID_TPS650333 = 0xF0; 

var reg_data = [];
var crc = 0x00;


function main() {
    /*
     * To debug this script, open the debugger in the main application window that
     * spawned this window. For most browser, press F12 to enable debugging.
     *
     * The 'debugger' keyword is an inline breakpoint. Uncomment the following line to stop
     * the script at main when the debugger is opened.
     */
    // debugger

    /* Log to text file */
    log(SCRIPT_START_MESSAGE);

	var old_crccode = 0xFF;
	var new_crccode;
	var pid = read('PID'); 

	reg_data[0] = read('LDO_GPIO_CFG');
	reg_data[1] = read('BUCK1_VOUT');
	reg_data[2] = read('BUCK1_UVLO');
	reg_data[3] = read('BUCK2_VOUT');
	reg_data[4] = read('BUCK3_VOUT');
	reg_data[5] = read('LDO_VOUT');
	reg_data[6] = read('DISCHARGE_SETTING');
	reg_data[7] = read('SEQ_TRIG_BUCK1');
	reg_data[8] = read('SEQ_TRIG_BUCK2');
	reg_data[9] = read('SEQ_TRIG_BUCK3');
	reg_data[10] = read('SEQ_TRIG_LDO');
	reg_data[11] = read('SEQ_TRIG_nRSTOUT');
	reg_data[12] = read('SEQ_TRIG_GPIO');
	reg_data[13] = read('BUCK1_SEQ_DLY');
	reg_data[14] = read('BUCK2_SEQ_DLY');
	reg_data[15] = read('BUCK3_SEQ_DLY');
	reg_data[16] = read('LDO_SEQ_DLY');
	reg_data[17] = read('nRSTOUT_SEQ_DLY');
	reg_data[18] = read('GPIO_SEQ_DLY');
	reg_data[19] = read('INT_MASK_BUCKx_LDO');
	reg_data[20] = read('VMON_BUCK1_CFG');
	reg_data[21] = read('VMON_BUCK2_CFG');
	reg_data[22] = read('VMON_BUCK3_CFG');
	reg_data[23] = read('VMON_LDO_CFG');
	reg_data[24] = read('ERR_BUCKx_LDO_CFG');
	reg_data[25] = read('INT_MASK_SAFETY');		
	reg_data[26] = read('PIN_MON_ERR_CFG');	
	
	if (pid >= PID_TPS650333) {
		reg_data[27] = read('WD_WIN1_CFG'); 		
		reg_data[28] = read('WD_WIN2_CFG'); 		
		reg_data[29] = read('WD_LONGWIN_CFG'); 		
		reg_data[30] = read('WD_MODE_REG') & 0x02; 		
		reg_data[31] = read('WD_QA_CFG'); 			
		reg_data[32] = read('WD_THR_CFG'); 
	}

	for(i=0;i<reg_data.length;i++) { 

         new_crccode =  Config_CRC_calc(reg_data[i], old_crccode); //function definition is below
         old_crccode = new_crccode;
	}

	write('CONFIG_LOCK',0xAA);
	write('CFG_REG_CRC', new_crccode);

	log(new_crccode); 

	/* Log to text file */
    log(SCRIPT_END_MESSAGE);

}

function Config_CRC_calc(data, crc)
{

    var d0=bit_select(data,1,1);//bit_select definition is below
    var d1=bit_select(data,2,2);
    var d2=bit_select(data,3,3);
    var d3=bit_select(data,4,4);
    var d4=bit_select(data,5,5);
    var d5=bit_select(data,6,6);
    var d6=bit_select(data,7,7);
    var d7=bit_select(data,8,8);

    var c0=bit_select(crc,1,1);
    var c1=bit_select(crc,2,2);
    var c2=bit_select(crc,3,3);
    var c3=bit_select(crc,4,4);
    var c4=bit_select(crc,5,5);
    var c5=bit_select(crc,6,6);
    var c6=bit_select(crc,7,7);
    var c7=bit_select(crc,8,8);

    var n0 = d7^d6^d0^c0^c6^c7;
    var n1 = d6^d1^d0^c0^c1^c6;
    var n2 = d6^d2^d1^d0^c0^c1^c2^c6;
    var n3 = d7^d3^d2^d1^c1^c2^c3^c7;
    var n4 = d4^d3^d2^c2^c3^c4;
    var n5 = d5^d4^d3^c3^c4^c5;
    var n6 = d6^d5^d4^c4^c5^c6;
    var n7 = d7^d6^d5^c5^c6^c7;

    crc = n7<<7; //n0 – n7 are global unsigned integer, initialized in the above Config_CRC_calc function
            crc += n6<<6;
            crc += n5<<5;
            crc += n4<<4;
            crc += n3<<3;
            crc += n2<<2;
            crc += n1<<1;
            crc += n0;

return crc;
 
}

function bit_select(num, start, end)
{
      //assert(end >= start);
      
      var mask = (1<< (end-start+1)) -1;
      var shift = start - 1;

      return (num & (mask << shift)) >> shift;

}